"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPassedAllChecks = exports.checkSuiteState = exports.getAutoMergeThresholdDays = exports.getTriggerWorkflowMessage = exports.isActiveTriggerWorkflow = exports.getApiToken = exports.isSetApiToken = exports.isCached = exports.getCache = exports.getCacheKey = exports.getActionContext = exports.ensureGetPulls = exports.getPullsArgsForDefaultBranch = exports.getHelper = exports.filterExtension = exports.filterGitStatus = exports.getGitFilterStatus = exports.isTargetContext = exports.isTargetBranch = exports.isNotCreatePR = exports.isClosePR = exports.isDisabledDeletePackage = exports.checkOnlyDefaultBranch = exports.checkDefaultBranch = exports.isDefaultBranch = exports.getContextBranch = exports.isActionPr = exports.getPrBranchPrefixForDefaultBranch = exports.getPrBranchPrefix = exports.getPrBaseRef = exports.getPrHeadRef = exports.getDefaultBranchUrl = exports.replaceDirectory = exports.getActionDetail = exports.ParameterRequiredError = void 0;
const core_1 = require("@actions/core");
const github_action_helper_1 = require("@technote-space/github-action-helper");
const github_action_log_helper_1 = require("@technote-space/github-action-log-helper");
const filter_github_action_1 = require("@technote-space/filter-github-action");
const command_1 = require("./command");
const constant_1 = require("../constant");
const { getWorkspace, getPrefixRegExp, getAccessToken } = github_action_helper_1.Utils;
const { escapeRegExp, replaceAll, getBranch } = github_action_helper_1.Utils;
const { isPr, isCron, isPush, isCustomEvent, isManualEvent, isWorkflowRun } = github_action_helper_1.ContextHelper;
/**
 * ParameterRequiredError
 */
class ParameterRequiredError extends Error {
    /**
     * @param {string} target target
     */
    constructor(target) {
        super(`parameter [${target}] is required.`);
        Object.setPrototypeOf(this, ParameterRequiredError.prototype);
    }
}
exports.ParameterRequiredError = ParameterRequiredError;
exports.getActionDetail = (key, context, defaultValue) => {
    if (undefined === defaultValue && !(key in context.actionDetail)) {
        throw new ParameterRequiredError(key);
    }
    if (undefined === defaultValue && typeof context.actionDetail[key] === 'string' && context.actionDetail[key].trim() === '') {
        throw new ParameterRequiredError(key);
    }
    return context.actionDetail[key] || (typeof defaultValue === 'function' ? defaultValue() : undefined);
};
const toArray = (item) => Array.isArray(item) ? item : [item];
exports.replaceDirectory = (message) => {
    const workDir = getWorkspace();
    return replaceAll(replaceAll(message, ` -C ${workDir}`, ''), workDir, '[Working Directory]');
};
exports.getDefaultBranchUrl = (octokit, context) => __awaiter(void 0, void 0, void 0, function* () { return `https://github.com/${context.actionContext.repo.owner}/${context.actionContext.repo.repo}/tree/${yield command_1.getDefaultBranch(octokit, context)}`; });
exports.getPrHeadRef = (context) => { var _a, _b; return (_b = (_a = context.actionContext.payload.pull_request) === null || _a === void 0 ? void 0 : _a.head.ref) !== null && _b !== void 0 ? _b : ''; };
exports.getPrBaseRef = (context) => { var _a, _b; return (_b = (_a = context.actionContext.payload.pull_request) === null || _a === void 0 ? void 0 : _a.base.ref) !== null && _b !== void 0 ? _b : ''; };
exports.getPrBranchPrefix = (context) => context.actionDetail.prBranchPrefix || `${context.actionDetail.actionRepo}/`;
exports.getPrBranchPrefixForDefaultBranch = (context) => context.actionDetail.prBranchPrefixForDefaultBranch || exports.getPrBranchPrefix(context);
exports.isActionPr = (context) => getPrefixRegExp(exports.getPrBranchPrefix(context)).test(exports.getPrHeadRef(context)) || getPrefixRegExp(exports.getPrBranchPrefixForDefaultBranch(context)).test(exports.getPrHeadRef(context));
exports.getContextBranch = (context) => context.isBatchProcess ? exports.getPrBaseRef(context) : (getBranch(context.actionContext) || exports.getPrHeadRef(context));
exports.isDefaultBranch = (octokit, context) => __awaiter(void 0, void 0, void 0, function* () { return (yield command_1.getDefaultBranch(octokit, context)) === exports.getContextBranch(context); });
exports.checkDefaultBranch = (context) => { var _a; return (_a = context.actionDetail.checkDefaultBranch) !== null && _a !== void 0 ? _a : true; };
exports.checkOnlyDefaultBranch = (context) => { var _a; return (_a = context.actionDetail.checkOnlyDefaultBranch) !== null && _a !== void 0 ? _a : false; };
exports.isDisabledDeletePackage = (context) => { var _a; return !((_a = context.actionDetail.deletePackage) !== null && _a !== void 0 ? _a : false); };
exports.isClosePR = (context) => isPr(context.actionContext) && context.actionContext.payload.action === 'closed';
exports.isNotCreatePR = (context) => isPr(context.actionContext) && true === context.actionDetail.notCreatePr;
exports.isTargetBranch = (branchName, octokit, context) => __awaiter(void 0, void 0, void 0, function* () {
    if (branchName === (yield command_1.getDefaultBranch(octokit, context))) {
        return exports.checkDefaultBranch(context);
    }
    const prefix = toArray(exports.getActionDetail('targetBranchPrefix', context, () => []));
    if (prefix.length) {
        return prefix.some(prefix => getPrefixRegExp(prefix).test(branchName));
    }
    return !exports.checkOnlyDefaultBranch(context);
});
exports.isTargetContext = (octokit, context) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    if (!filter_github_action_1.isTargetEvent((_a = context.actionDetail.targetEvents) !== null && _a !== void 0 ? _a : constant_1.DEFAULT_TARGET_EVENTS, context.actionContext)) {
        return false;
    }
    if (isCron(context.actionContext) || isCustomEvent(context.actionContext) || isManualEvent(context.actionContext) || isWorkflowRun(context.actionContext)) {
        return true;
    }
    if (isPush(context.actionContext)) {
        return exports.isTargetBranch(getBranch(context.actionContext), octokit, context);
    }
    if (exports.isActionPr(context)) {
        return true;
    }
    if (exports.isClosePR(context)) {
        return true;
    }
    if (!(yield exports.isTargetBranch(exports.getPrHeadRef(context), octokit, context))) {
        return false;
    }
    return filter_github_action_1.isTargetLabels(toArray(exports.getActionDetail('includeLabels', context, () => [])), [], context.actionContext);
});
exports.getGitFilterStatus = (context) => context.actionDetail.filterGitStatus;
exports.filterGitStatus = (line, context) => {
    const filter = exports.getGitFilterStatus(context);
    if (filter) {
        const targets = filter.toUpperCase().replace(/[^MDA]/g, '');
        if (!targets) {
            throw new Error('Invalid input [FILTER_GIT_STATUS].');
        }
        // language=JSRegexp
        return (new RegExp(`^[${targets}]\\s+`)).test(line);
    }
    return true;
};
exports.filterExtension = (line, context) => {
    const extensions = toArray(exports.getActionDetail('filterExtensions', context, () => []));
    if (extensions.length) {
        const pattern = '(' + extensions.map(item => escapeRegExp('.' + item.replace(/^\./, ''))).join('|') + ')';
        return (new RegExp(`${pattern}$`)).test(line);
    }
    return true;
};
exports.getHelper = (context) => new github_action_helper_1.GitHelper(new github_action_log_helper_1.Logger(exports.replaceDirectory), {
    depth: -1,
    filter: (line) => exports.filterGitStatus(line, context) && exports.filterExtension(line, context),
});
exports.getPullsArgsForDefaultBranch = (octokit, context) => __awaiter(void 0, void 0, void 0, function* () {
    return ({
        number: 0,
        id: 0,
        head: {
            ref: yield command_1.getDefaultBranch(octokit, context),
            user: {
                login: context.actionContext.repo.owner,
            },
        },
        base: {
            repo: {
                name: context.actionContext.repo.repo,
                owner: {
                    login: context.actionContext.repo.owner,
                },
            },
            ref: yield command_1.getDefaultBranch(octokit, context),
        },
        title: 'default branch',
        'html_url': yield exports.getDefaultBranchUrl(octokit, context),
    });
});
exports.ensureGetPulls = (pull, octokit, context) => __awaiter(void 0, void 0, void 0, function* () { return pull !== null && pull !== void 0 ? pull : yield exports.getPullsArgsForDefaultBranch(octokit, context); });
exports.getActionContext = (pull, octokit, context) => __awaiter(void 0, void 0, void 0, function* () {
    const _pull = yield exports.ensureGetPulls(pull, octokit, context);
    return Object.assign(Object.assign({}, context), { actionContext: Object.assign({}, context.actionContext, {
            payload: {
                number: _pull.number,
                'pull_request': {
                    number: _pull.number,
                    id: _pull.id,
                    head: _pull.head,
                    base: _pull.base,
                    title: _pull.title,
                    'html_url': _pull.html_url,
                },
            },
            repo: {
                owner: _pull.base.repo.owner.login,
                repo: _pull.base.repo.name,
            },
            ref: `refs/heads/${_pull.head.ref}`,
        }), isBatchProcess: !!_pull.number });
});
exports.getCacheKey = (method, args = {}) => method + JSON.stringify(args);
exports.getCache = (key, generator, context) => __awaiter(void 0, void 0, void 0, function* () {
    if (!(key in context.cache)) {
        // eslint-disable-next-line require-atomic-updates
        context.cache[key] = yield generator();
    }
    return context.cache[key];
});
exports.isCached = (key, context) => key in context.cache;
exports.isSetApiToken = () => !!core_1.getInput('API_TOKEN');
exports.getApiToken = () => core_1.getInput('API_TOKEN') || getAccessToken(true);
exports.isActiveTriggerWorkflow = (context) => exports.isSetApiToken() && '' !== context.actionDetail.triggerWorkflowMessage;
exports.getTriggerWorkflowMessage = (context) => { var _a; return (_a = context.actionDetail.triggerWorkflowMessage) !== null && _a !== void 0 ? _a : constant_1.DEFAULT_TRIGGER_WORKFLOW_MESSAGE; };
// eslint-disable-next-line no-magic-numbers
exports.getAutoMergeThresholdDays = (context) => context.actionDetail.autoMergeThresholdDays && /^\d+$/.test(context.actionDetail.autoMergeThresholdDays) ? Number(context.actionDetail.autoMergeThresholdDays) : 0;
exports.checkSuiteState = (checkSuiteId) => (suite) => {
    if (suite.conclusion === 'success') {
        return false;
    }
    if (suite.status !== 'in_progress' || suite.app.slug !== 'github-actions') {
        return true;
    }
    return suite.id !== checkSuiteId;
};
exports.isPassedAllChecks = (octokit, context) => __awaiter(void 0, void 0, void 0, function* () {
    const { data: status } = yield octokit.repos.getCombinedStatusForRef(Object.assign(Object.assign({}, context.actionContext.repo), { ref: context.actionContext.sha }));
    if ('success' !== status.state) {
        return false;
    }
    const checkSuiteUrl = (yield octokit.actions.getWorkflowRun(Object.assign(Object.assign({}, context.actionContext.repo), { 'run_id': Number(process.env.GITHUB_RUN_ID) }))).data['check_suite_url'];
    const checkSuiteId = Number(checkSuiteUrl.replace(/^.+\/(\d+)$/, '$1'));
    return !(yield octokit.paginate(octokit.checks.listSuitesForRef.endpoint.merge(Object.assign(Object.assign({}, context.actionContext.repo), { ref: context.actionContext.sha })))).filter(suite => exports.checkSuiteState(checkSuiteId)(suite)).length;
});
